package in;

import java.io.BufferedInputStream;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Random;

//Sie erhalten eine HTML-Dokumentation dieser Klasse, 
//wenn Sie den Befehl "javadoc Input.java" in dem Verzeichnis 
//dieser Klasse ausfhren. ffnen Sie die damit erzeugte 
//Datei "index.html" in Ihrem Browser

/** Klassenmethoden zur einfachen Eingabe und Erzeugung von Daten in java.
 * Die Daten knnen entweder ber die Tastatur eingegeben oder
 * zufllig generiert werden. <p>
 * Bisher werden die folgenden, primitiven Datentypen untersttzt:
 * int, long, float, double, char, boolean;<p>
 * Sowie die folgenden Referenztypen: String, String[], int[], float[], char[], boolean[]<p>
 * Darber hinaus knnen einfache Text- oder CSV-Dateien eingelesen werden;
 * Texte werden als String[] zurck  gegeben (aufgeteilt nach Zeilenumbrchen);
 * csv-Dateien werden als String[][] zurck  gegeben (aufgeteilt nach ";" und Zeilenumbrchen);
  * <p>
 * @author Marcus Deininger
 * @version 2.4 / 28.03.2012
 */

public class Input {
	
	private static Random Generator = new Random();
	
	private Input(){
		//keine explizite Erzeugung der Klasse
	}

	/**
	 * Liest eine Zeile als String von der Tastatur ein;
	 * Die Eingabe wird durch &lt;return&gt; beendet.
	 * @return Die eingegebene Zeile
	 */
	public static String readString(){
		InputStreamReader in = new InputStreamReader(System.in);
		BufferedReader buffer = new BufferedReader(in);
		String input = "";
		try {
			input = buffer.readLine();
		} catch ( IOException e) {
			e.printStackTrace();
		}
		return input;
	}
	
	/**
	 * Liest eine Zeile von der Tastatur ein und zerlegt die Eingabe 
	 * an den Leerrumen in Teilstrings. Die Eingabe wird durch &lt;return&gt; beendet.
	 * @return Ein Feld mit den eigegebenen Strings
	 */
	public static String[] readStringVector(){
		String input = readString();
		String[] tokens = input.split(" +");
		return tokens;
	}
	
	
	/**
	 * Wandelt einen <tt>String</tt> in einen <tt>int</tt>-Wert um.
	 */
	private static int toInt(String input){
		int value = Integer.parseInt(input);
		return value;		
	}

	/**
	 * Liest einen <tt>int</tt>-Wert von der Tastatur ein.
	 * Die Eingabe wird durch &lt;return&gt; beendet.
	 * @return Der eingelesene Wert
	 */
	public static int readInt(){
		String input = readString();
		int value = toInt(input);
		return value;
	}

	
	
	/**
	 * Wandelt einen <tt>String</tt> in einen <tt>Long</tt>-Wert um.
	 */
	private static Long toLong(String input){
		Long value = Long.parseLong(input);
		return value;		
	}

	/**
	 * Liest einen <tt>Long</tt>-Wert von der Tastatur ein.
	 * Die Eingabe wird durch &lt;return&gt; beendet.
	 * @return Der eingelesene Wert
	 */
	public static Long readLong(){
		String input = readString();
		Long value = toLong(input);
		return value;
	}

	/**
	 * Liest einen <tt>int</tt>-Vektor von der Tastatur ein. 
	 * Die einzelnen Werte werden durch Leerzeichen getrennt. 
	 * Die Eingabe wird durch &lt;return&gt; beendet.
	 * @return Der eingelesene Vektor
	 */
	public static int[] readIntVector(){
		String[] tokens = readStringVector();
		int[] vector = new int[tokens.length];
		for(int i = 0; i < vector.length;i++)
			vector[i] = toInt(tokens[i]);
		return vector;
	}

	/**
	 * Erzeugt einen zuflligen <tt>int</tt>-Wert.
	 * @param von untere Grenze
	 * @param bis obere Grenze
	 * @return Der erzeugte Wert
	 */
	public static int randomInt(int von, int bis){
		int bereich = bis - von + 1;			// normierter Bereich
		int zahl = Generator.nextInt(bereich);	// Zufallszahl zwischen 0 und bereich - 1
		int ergebnis = bis - zahl;				// zurck transformieren
		return ergebnis;
	}

	/**
	 * Erzeugt einen zufllig besetzen <tt>int</tt>-Vektor mit der vorgegebenen Lnge.
	 * @param laenge die Lnge des zu erzeugenden Vektors
	 * @param von untere Grenze der Zufallszahlen
	 * @param bis obere Grenze der Zufallszahlen
	 * @return Der erzeugte Vektor
	 */
	public static int[] randomIntVector(int laenge, int von, int bis){
		int[] result = new int[laenge];
		for (int i = 0; i < laenge; i++)
			result[i] = randomInt(von, bis);
		return result;
	}

	/**
	 * Wandelt einen <tt>String</tt> in einen <tt>float</tt>-Wert um.
	 */
	private static float toFloat(String input){
		float value = Float.parseFloat(input);
		return value;
	}

	/**
	 * Liest einen <tt>float</tt>-Wert von der Tastatur ein.
	 * Die Eingabe wird durch &lt;return&gt; beendet.
	 * @return Der eingelesene Wert
	 */
	public static float readFloat(){
		String input = readString();
		float value = toFloat(input);
		return value;
	}

	/**
	 * Liest einen <tt>float</tt>-Vektor von der Tastatur ein. 
	 * Die einzelnen Werte werden durch Leerzeichen getrennt. 
	 * Die Eingabe wird durch &lt;return&gt; beendet.
	 * @return Der eingelesene Vektor
	 */
	public static float[] readFloatVector(){
		String[] tokens = readStringVector();
		float[] vector = new float[tokens.length];
		for(int i = 0; i < vector.length;i++)
			vector[i] = toFloat(tokens[i]);
		return vector;
	}

	/**
	 * Erzeugt einen zuflligen <tt>float</tt>-Wert.
	 * @param von untere Grenze
	 * @param bis obere Grenze
	 * @return Der erzeugte Wert
	 */
	public static float randomFloat(float von, float bis){
		float zahl = Generator.nextFloat(); 		// Zufallszahl zwischen 0.0 und 1.0
		float ergebnis = zahl * (bis - von) + von;	// auf Bereich transformieren
		return ergebnis;
	}

	/**
	 * Erzeugt einen zufllig besetzen <tt>float</tt>-Vektor mit der vorgegebenen Lnge.
	 * @param laenge die Lnge des zu erzeugenden Vektors
	 * @param von untere Grenze der Zufallszahlen
	 * @param bis obere Grenze der Zufallszahlen
	 * @return Der erzeugte Vektor
	 */
	public static float[] randomFloatVector(int laenge, float von, float bis){
		float[] result = new float[laenge];
		for (int i = 0; i < laenge; i++)
			result[i] = randomFloat(von, bis);
		return result;
	}

	/**
	 * Wandelt einen <tt>String</tt> in einen <tt>double</tt>-Wert um.
	 */
	private static double toDouble(String input){
		double value = Double.parseDouble(input);
		return value;
	}

	/**
	 * Liest einen <tt>double</tt>-Wert von der Tastatur ein.
	 * Die Eingabe wird durch &lt;return&gt; beendet.
	 * @return Der eingelesene Wert
	 */
	public static double readDouble(){
		String input = readString();
		double value = toDouble(input);
		return value;
	}

	/**
	 * Liest einen <tt>double</tt>-Vektor von der Tastatur ein. 
	 * Die einzelnen Werte werden durch Leerzeichen getrennt. 
	 * Die Eingabe wird durch &lt;return&gt; beendet.
	 * @return Der eingelesene Vektor
	 */
	public static double[] readDoubleVector(){
		String[] tokens = readStringVector();
		double[] vector = new double[tokens.length];
		for(int i = 0; i < vector.length;i++)
			vector[i] = toDouble(tokens[i]);
		return vector;
	}

	/**
	 * Erzeugt einen zuflligen <tt>double</tt>-Wert.
	 * @param von untere Grenze
	 * @param bis obere Grenze
	 * @return Der erzeugte Wert
	 */
	public static double randomDouble(double von, double bis){
		double zahl = Generator.nextDouble(); 		// Zufallszahl zwischen 0.0 und 1.0
		double ergebnis = zahl * (bis - von) + von;	// auf Bereich transformieren
		return ergebnis;
	}

	/**
	 * Erzeugt einen zufllig besetzen <tt>double</tt>-Vektor mit der vorgegebenen Lnge.
	 * @param laenge die Lnge des zu erzeugenden Vektors
	 * @param von untere Grenze der Zufallszahlen
	 * @param bis obere Grenze der Zufallszahlen
	 * @return Der erzeugte Vektor
	 */
	public static double[] randomDoubleVector(int laenge, double von, double bis){
		double[] result = new double[laenge];
		for (int i = 0; i < laenge; i++)
			result[i] = randomDouble(von, bis);
		return result;
	}


	
	/**
	 * Wandelt einen <tt>String</tt> in einen <tt>char</tt>-Wert um.
	 * Dabei wird das erste Zeichen des Strings genommen. Ist der
	 * String lnger als ein Zeichen, wird eine Exception ausgelst.
	 */
	private static char toChar(String input){
		if (input.length() != 1) throw new RuntimeException();
		char value = input.charAt(0);
		return value;
	}

	/**
	 * Liest einen <tt>char</tt>-Wert von der Tastatur ein.
	 * Die Eingabe wird durch &lt;return&gt; beendet.
	 * @return Der eingelesene Wert
	 */
	public static char readChar(){
		String input = readString();
		char value = toChar(input);
		return value;
	}
	
	/**
	 * Liest einen <tt>char</tt>-Vektor von der Tastatur ein. 
	 * Die einzelnen Werte werden durch Leerzeichen getrennt. 
	 * Die Eingabe wird durch &lt;return&gt; beendet.
	 * @return Der eingelesene Vektor
	 */
	public static char[] readCharVector(){
		String[] tokens = readStringVector();
		char[] vector = new char[tokens.length];
		for(int i = 0; i < vector.length;i++)
			vector[i] = toChar(tokens[i]);
		return vector;
	}

	/**
	 * Erzeugt einen zuflligen <tt>char</tt>-Wert zwischen ASCII(33) und ASCII(127).
	 * Das sind die sichtbaren ASCII-Zeichen ohne Zeilenumbruch.
	 * @return Der erzeugte Wert
	 */
	public static char randomChar(){
		//Sichtbare Zeichen,ohne Zeilenumbruch
		return (char)randomInt(33, 126);
	}

	/**
	 * Erzeugt einen zufllig besetzen <tt>char</tt>-Vektor mit der vorgegebenen Lnge.
	 * Die Werte liegen zwischen ASCII(33) und ASCII(127).
	 * Das sind die sichtbaren ASCII-Zeichen ohne Zeilenumbruch.
	 * @param laenge die Lnge des zu erzeugenden Vektors
	 * @return Der erzeugte Vektor
	 */
	public static char[] randomCharVector(int laenge){
		char[] result = new char[laenge];
		for (int i = 0; i < laenge; i++)
			result[i] = randomChar();
		return result;
	}
	

	/**
	 * Wandelt einen <tt>String</tt> in einen <tt>boolean</tt>-Wert um.
	 * Mgliche Eingaben sind <tt>t</tt>, <tt>true</tt>, <tt>f</tt>, 
	 * oder <tt>false</tt> (unabhngig von Gro- und Kleinschreibung).
	 */
	private static boolean toBoolean(String input){		
		if(input.equalsIgnoreCase("t")) return true;
		if(input.equalsIgnoreCase("true")) return true;
		if(input.equalsIgnoreCase("f")) return false;
		if(input.equalsIgnoreCase("false")) return false;

		throw new RuntimeException();
	}

	/**
	 * Liest einen <tt>boolean</tt>-Wert von der Tastatur ein.
	 * Boolsche Werte knnen durch <tt>t</tt>, <tt>true</tt>, <tt>f</tt>, 
	 * oder <tt>false</tt> (unabhngig
	 * von Gro- und Kleinschreibung) eingegeben werden.
	 * Die Eingabe wird durch &lt;return&gt; beendet.
	 * @return Der eingelesene Wert
	 */
	public static boolean readBoolean(){
		String input = readString();
		boolean value = toBoolean(input);
		return value;
	}
	
	/**
	 * Liest einen <tt>boolean</tt>-Vektor von der Tastatur ein. 
	 * Boolsche Werte knnen durch <tt>t</tt>, <tt>true</tt>, <tt>f</tt>, 
	 * oder <tt>false</tt> (unabhngig
	 * von Gro- und Kleinschreibung) eingegeben werden.
	 * Die einzelnen Werte werden durch Leerzeichen getrennt. 
	 * Die Eingabe wird durch &lt;return&gt; beendet.
	 * @return Der eingelesene Vektor
	 */
	public static boolean[] readBooleanVector(){
		String[] tokens = readStringVector();
		boolean[] vector = new boolean[tokens.length];
		for(int i = 0; i < vector.length;i++)
			vector[i] = toBoolean(tokens[i]);
		return vector;
	}

	
	/**
	 * Erzeugt einen zuflligen <tt>boolean</tt>-Wert.
	 * @return Der erzeugte Wert
	 */
	public static boolean randomBoolean(){
		return randomInt(0, 1) == 1;
	}

	/**
	 * Erzeugt einen zufllig besetzten <tt>booelan</tt>-Vektor mit der vorgegebenen Lnge.
	 * @param laenge die Lnge des zu erzeugenden Vektors
	 * @return Der erzeugte Vektor
	 */
	public static boolean[] randomBooleanVector(int laenge){
		boolean[] result = new boolean[laenge];
		for (int i = 0; i < laenge; i++)
			result[i] = randomBoolean();
		return result;
	}
	
	/**
	 * Liest eine Textdatei mit dem eingegebenen Pfad und Namen
	 * liefert sie in einem Stringfeld zurck, aufgeteilt nach
	 * Zeilenumbrchen.
	 * @param filename Pfad und Name der Textdatei
	 * @return Inhalt der Textdatei, aufgeteilt nach Zeilenumbrchen
	 */
	public static String[] readTextFile(String filename){

		ArrayList<String> lines = new ArrayList<String>();
		
		try {
			InputStream stream = Input.class.getClassLoader().getResourceAsStream(filename);
			if(stream == null)
				throw new IOException(filename + " not found");
	
			BufferedInputStream buffer = new BufferedInputStream(stream);
			int n = buffer.read();
			StringBuffer line = new StringBuffer();
			while(n != -1){
				char c = (char)n;
				if(c == '\n'){
					lines.add(line.toString());
					line = new StringBuffer();
				} else if(c != '\r') // skip \r
					line.append(c);
				n = buffer.read();
			}
			//flush last line
			if(line.length() > 0)
				lines.add(line.toString());
			buffer.close();

		} catch (IOException ex) {
			throw new RuntimeException(ex.getMessage());
		}					
			
		return lines.toArray(new String[lines.size()]);
	}
	
	
	/**
	 * Liest eine CSV-Datei mit dem eingegebenen Pfad und Namen
	 * liefert sie in einem Stringfeld zurck, aufgeteilt nach
	 * ";" und Zeilenumbrchen.
	 * @param filename Pfad und Name der Textdatei
	 * @return Inhalt der Textdatei, aufgeteilt nach ";" und Zeilenumbrchen
	 */
	public static String[][] readCsvFile(String filename){
		String[] readLines		= readTextFile(filename);
		String[][] splitLines	= new String[readLines.length][];
		
		for(int i = 0; i < readLines.length; i++){
			String[] splitLine		= readLines[i].split(";");
			String[] trimmedLine	= new String[splitLine.length];
			for(int k = 0; k < splitLine.length; k++)
				trimmedLine[k] = splitLine[k].trim();
			
			splitLines[i] = trimmedLine;
		}
		
		return splitLines;
	}
	
	/**
	 * Liest eine CSV-Datei mit dem eingegebenen Pfad und Namen
	 * liefert sie in einem Stringfeld zurck, aufgeteilt nach
	 * ";" und Zeilenumbrchen.
	 * @param filename Pfad und Name der Textdatei
	 * @param readHeader true, falls die erste Zeile (mit Header-Informationen) ebenfalls zurck
	 * gegeben werden soll, false, falls nicht.
	 * @return Inhalt der Textdatei, aufgeteilt nach ";" und Zeilenumbrchen
	 */
	public static String[][] readCsvFile(String filename, boolean readHeader){
		String[][] lines = readCsvFile(filename);
		if(lines.length == 0)
			return lines;
		return Arrays.copyOfRange(lines, 1, lines.length);
	}
}

